#include "CATDMUsrMethod.h"
#include <iostream.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "ISPPS9DMUsrMethod.h"

// This example shows how to call external routines (see the 
// DuplicateRepresentations method below)
// ----------------------------------------------------------
// Technical explanation:
//   C++ compilers mangle (transform) method names into
//  proprietary C-style function names, according to the 
//  class name, method name, parameters, etc.
//  When you call external routines that were written in C or
//  Fortran, you do not want the compiler to mangle the names
//  of your functions, since if it did, it would not be able
//  to link to the correct functions either at link-time (AIX)
//  or at run-time(SGI, SUN).  To inhibit the mangling of
//  these functions' names, they must be declared as 
//  'extern "C"' functions.  Beware, because standard C
//  functions such as strcpy() or fopen() have already been
//  declared this way in the required headers, so they do NOT
//  need to be re-declared.
// ----------------------------------------------------------
//
// In this example, we will retrieve the value of an environment variable
// (USER, for example), using both the catgeo routine 'g7aget' and the standard
// C function 'getenv'.  Notice that 'g7aget' was declared with 'extern "C"' (see top).
//
// CATGEO method
// =============
//
//  extern "C" int g7aget(const char* var, const int* lvar,
//                       char* val, int* lval, int* ier);
//
//         **************
//
//  char envvar[255];
//  memset(envvar,' ',255);
//  memcpy(envvar,"USER",4);
//  int lenvvar = 8;
//  char envvarval[9];
//  int lenvvarval = 0;
//  int ier = 0;
//  g7aget(envvar,&lenvvar,envvarval,&lenvvarval,&ier);
//  if ( ier != 0 )
//    cout << "CATPSNUsrMethod::DuplicateRepresentations - ier = " << ier << endl;
//  else
//  {
//    envvarval[lenvvarval] = 0;
//    cout << "Hello, my name is " << envvarval << "." << endl;
//  }
// 
// Std C function method
// =====================
//
//  cout << "Hello again, my name is really " << getenv("USER") << "." << endl;
//


Error CATDMUsrMethod::ExecuteUsrMethod(const char *iMethodName,
				       const int &iNbSelectedInstances,
				       ObjID *iSelectedObjects)
{
  Error ier;
  ier.Ier=0;
  ier.ShortMsg=NULL;
  ier.LongMsg=NULL;
  //--------------------------
  if(getenv("TRACEUSRMETHOD"))
    {
      printf("************************************************************\n");
      printf("Method name : %s\n",iMethodName);
      printf("Nb objects : %i\n",iNbSelectedInstances);
      printf("************************************************************\n");
    }
  //--------------------------
  return ier;
}


Error CATDMUsrMethod::OpenPSNWithUsrMethod(const char* iMethodName,
					   const int& iNbSelectedInstances,
					   int* iDepthArray,
					   PartAsmID** iObjArray,
					   double** iComputedXmatArray)
{ 
  Error ier;
  ier.Ier=0;
  ier.ShortMsg=NULL;
  ier.LongMsg=NULL;
  //--------------------------
  if(getenv("TRACEUSRMETHOD"))
    {
      printf("**** iMethodName = [%s] ****\n",iMethodName);
      printf("**** iNbSelectedInstances = %i ****\n",iNbSelectedInstances);
      for (int j=0;j<iNbSelectedInstances;j++) 
	{
	  printf("  **** iObjArray = %i      iDepthArray[%i] = %i ****\n",
		 j,j,iDepthArray[j]);
	  for (int i=0;i<iDepthArray[j];i++)
	    {
	      printf("      **** Obj = %i ****\n",i);
	      printf("           Coid   = [%8.8s]\n",iObjArray[j][i].coid);
	      printf("           Compid = [%8.8s]\n",iObjArray[j][i].compid);
	      printf("           Caenv  = [%8.8s]\n",iObjArray[j][i].caenv);
	      printf("           Catab  = [%18.18s]\n",iObjArray[j][i].catab);
	      printf("           CLink  = [%8.8s]\n",iObjArray[j][i].clien);
	      printf("           Depth  = %i\n",iObjArray[j][i].level);
	      printf("           %lf  %lf  %lf\n",iObjArray[j][i].xmat[0],iObjArray[j][i].xmat[1],iObjArray[j][i].xmat[2]);
	      printf("           %lf  %lf  %lf\n",iObjArray[j][i].xmat[3],iObjArray[j][i].xmat[4],iObjArray[j][i].xmat[5]);
	      printf("           %lf  %lf  %lf\n",iObjArray[j][i].xmat[6],iObjArray[j][i].xmat[7],iObjArray[j][i].xmat[8]);
	      printf("           %lf  %lf  %lf\n",iObjArray[j][i].xmat[9],iObjArray[j][i].xmat[10],iObjArray[j][i].xmat[11]);
	    }
	  
	  printf("  **** iComputedXmatArray ****\n");
	  printf("           %lf  %lf  %lf\n",iComputedXmatArray[j][0],iComputedXmatArray[j][1],iComputedXmatArray[j][2]);
	  printf("           %lf  %lf  %lf\n",iComputedXmatArray[j][3],iComputedXmatArray[j][4],iComputedXmatArray[j][5]);
	  printf("           %lf  %lf  %lf\n",iComputedXmatArray[j][6],iComputedXmatArray[j][7],iComputedXmatArray[j][8]);
	  printf("           %lf  %lf  %lf\n",iComputedXmatArray[j][9],iComputedXmatArray[j][10],iComputedXmatArray[j][11]);
	}
    }
  //--------------------------
  return ier;
}


void CATDMUsrMethod::GetConfNameHistory(const char iUser[9], const char iOrg[11],
                                        char*** oHistory, int* oHistSize)
{
  *oHistory = NULL;
  *oHistSize = 0;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("GetConfNameHistory\n");
    }
}

Error CATDMUsrMethod::IsAValidFlagForCommit(const char* iModificationFlag,
					    const int iNbModifications,
					    int* iTypeOfModifications,
					    ObjID* iAddedLinks,
					    ObjID* iRemovedLinks)
{
  Error ier;    
  ier.Ier=0;
  ier.ShortMsg = NULL;
  ier.LongMsg = NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("IsAValidFlagForCommit\n");
    }

  return ier;
}

Error CATDMUsrMethod::IsAValidParamListForCommit(const int iNbParam, EffParam *iParamPtr,
						 const int iNbModifications,
						 int* iTypeOfModifications,
						 ObjID* iAddedLinks, ObjID* iRemovedLinks)
{
  Error ier;    
  ier.Ier=0;
  ier.ShortMsg = NULL;
  ier.LongMsg = NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("IsAValidParamListForCommit\n");
    }

  return ier;
}

Error CATDMUsrMethod::IsExistingParamValidForCommit(const int iNbModifications,
						    int* iTypeOfModifications,
						    ObjID* iAddedLinks, ObjID* iRemovedLinks)
{
  Error ier;    
  ier.Ier=0;
  ier.ShortMsg = NULL;
  ier.LongMsg = NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("IsExistingParamValidForCommit\n");
    }

  return ier;
}

int CATDMUsrMethod::ActionFlowCheck(const char *iModificationFlag, const char *iUser)
{
  int ret = 1;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("ActionFlowCheck(%s,%s)\n",iModificationFlag,iUser);
    }
  // In this sample, we suppose that the rule is:
  // If the modification flag begins with DEV, then don't call the Action Flow. 
  if (strlen(iModificationFlag) >=3 && 
      !memcmp(iModificationFlag, "DEV", 3)) ret = 0;

  return ret;
}

Error CATDMUsrMethod::SuccessfullyCommitInGraph(const char* iModificationFlag,
						     const int iNbModifications,
						     int* iTypeOfModifications,
						     ObjID* iAddedLinks,
						     ObjID* iRemovedLinks)
{
  Error ier;    
  ier.Ier=0;
  ier.ShortMsg = NULL;
  ier.LongMsg = NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("SuccessfullyCommitInGraph(%i) BEGIN\n",iNbModifications);
      for(int i=0;i<iNbModifications;i++)
	{
	  if(iTypeOfModifications[i]&ADDLINK)
	    printf ("(%i) ADDLINK :\n",i+1);
	  if(iTypeOfModifications[i]&REMOVELINK)
	    printf ("(%i) REMOVELINK :\n",i+1);
	  if(iTypeOfModifications[i]&MOVELINK)
	    printf ("(%i) MOVELINK :\n",i+1);
	  if(iTypeOfModifications[i]&REPLACELINK)
	    printf ("(%i) REPLACELINK :\n",i+1);
	  if(iTypeOfModifications[i]&RENAMELINK)
	    printf ("(%i) RENAMELINK :\n",i+1);
	  
	  if(iAddedLinks[i].coid)
	    printf ("[%s], [%s], [%s], [%s]\n",
		    iAddedLinks[i].caenv, iAddedLinks[i].catab, 
		    iAddedLinks[i].Xcoid, iAddedLinks[i].Xcompid);
	  if(iRemovedLinks[i].coid)
	    printf ("[%s], [%s], [%s], [%s]\n",
		    iRemovedLinks[i].caenv, iRemovedLinks[i].catab, 
		    iRemovedLinks[i].Xcoid, iRemovedLinks[i].Xcompid);
	}
      printf ("SuccessfullyCommitInGraph(%i) END\n",iNbModifications);
    }
  //--------------------------
  return ier;
}

void CATDMUsrMethod::SuccessfullyRollBackInGraph()
{
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("SuccessfullyRollBackInGraph\n");
    }
}

Error CATDMUsrMethod::DefaultModValidation(const char *iModif,
					    char **oType,
					    int &oOptListSize,
					    char ***oOptList,
					    int &oRangeBot, int &oRangeTop,
					    time_t &oDateBeg, time_t &oDateEnd)
{
  Error ier;    
  ier.Ier=0;
  ier.ShortMsg = NULL;
  ier.LongMsg = NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("DefaultModValidation\n");
    }
  return ier;
}

Error CATDMUsrMethod::AuthorizedModValidation(const char *iModification,
					      const char *iUser,                               
					      const char *iValidationType,
					      const int iNbOptions,
					      char **iOptionList,
					      const int iRangeBot,   const int iRangeTop,
					      const time_t iDateBeg, const time_t iDateEnd)
{
  Error ier;    
  ier.Ier=0;
  ier.ShortMsg = NULL;
  ier.LongMsg = NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("AuthorizedModValidation\n");
    }
  return ier;
}

Error CATDMUsrMethod::AuthorizedModPromote(const char *iModification, const char *iUser)
{
  Error ier;    
  ier.Ier=0;
  ier.ShortMsg = NULL;
  ier.LongMsg = NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("AuthorizedModPromote\n");
    }
  return ier;
}                               

Error CATDMUsrMethod::AuthorizedModDemote(const char *iModification, const char *iUser)
{
  Error ier;    
  ier.Ier=0;
  ier.ShortMsg = NULL;
  ier.LongMsg = NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("AuthorizedModDemote\n");
    }
  return ier;
}                            

Error CATDMUsrMethod::AuthorizedModDelete(const char *iModification, const char *iUser)
{
  Error ier;    
  ier.Ier=0;
  ier.ShortMsg = NULL;
  ier.LongMsg = NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("AuthorizedModDelete\n");
    }
  return ier;
} 

int CATDMUsrMethod::ModCreateOwner(const char *iModification, const char *iUser)                                
{
    if(getenv("TRACEUSRMETHOD")) printf ("AuthorizedModDelete\n");

    // In this sample, we suppose that the rule is:
    // If the modification flag begins with DEVPUB, 
    // then the mod should be of public type. 
    int ret = 0;
    if (strlen(iModification) >=6 )
        if (!memcmp(iModification, "DEVPUB", 6)) ret = 1;
  return ret;
}

Error CATDMUsrMethod::AuthorizedCopyInstance(const int &NbSelectedInstances, ObjID * iSelectedInstances)
{
  Error ier;
  ier.Ier=0;
  ier.ShortMsg=NULL;
  ier.LongMsg=NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("AuthorizedCopyInstance\n");
      printf ("**** Nb selected objects: %i ****\n", NbSelectedInstances);
      for (int i=0;i<NbSelectedInstances;i++)
	{
	  printf("      **** Obj = %i ****\n",i);
	  printf("           Coid   = [%16.16s]\n",iSelectedInstances[i].Xcoid);
	  printf("           Compid = [%16.16s]\n",iSelectedInstances[i].Xcompid);
	  printf("           Caenv  = [%8.8s]\n",iSelectedInstances[i].caenv);
	  printf("           Catab  = [%18.18s]\n",iSelectedInstances[i].catab);
	}
    }
  //--------------------------
  return ier;
}

Error CATDMUsrMethod::AuthorizedCut(const ObjID& ilink)
{
  Error ier;
  ier.Ier=0;
  ier.ShortMsg=NULL;
  ier.LongMsg=NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("AuthorizedCut [%s], [%s], [%s], [%s]\n",
	      ilink.caenv, ilink.catab, ilink.Xcoid, ilink.Xcompid);
    }
  
  //  ier.Ier=1111;
  //  ier.ShortMsg=strdup("Short message!");
  //  ier.LongMsg=strdup("Long message!");
  //--------------------------
  return ier;
}

Error CATDMUsrMethod::AuthorizedCutInstance(const int & NbSelectedInstances, ObjID * iSelectedInstances)
{
  Error ier;
  ier.Ier=0;
  ier.ShortMsg=NULL;
  ier.LongMsg=NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("AuthorizedCutInstance\n");
      printf ("**** Nb selected objects: %i ****\n", NbSelectedInstances);
      for (int i=0;i<NbSelectedInstances;i++)
	{
	  printf("      **** Obj = %i ****\n",i);
	  printf("           Coid   = [%16.16s]\n",iSelectedInstances[i].Xcoid);
	  printf("           Compid = [%16.16s]\n",iSelectedInstances[i].Xcompid);
	  printf("           Caenv  = [%8.8s]\n",iSelectedInstances[i].caenv);
	  printf("           Catab  = [%18.18s]\n",iSelectedInstances[i].catab);
	}
    }
  //--------------------------
  return ier;
}

Error CATDMUsrMethod::AuthorizedPaste(const ObjID& ifather, const ObjID& ichild)
{
  Error ier;
  ier.Ier=0;
  ier.ShortMsg=NULL;
  ier.LongMsg=NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("AuthorizedPaste\n[%s], [%s], [%s], [%s]\n[%s], [%s], [%s], [%s]\n",
	      ifather.caenv, ifather.catab, ifather.Xcoid, ifather.Xcompid,
	      ichild.caenv, ichild.catab, ichild.Xcoid, ichild.Xcompid);
    }
  //--------------------------
  return ier;
}

Error CATDMUsrMethod::AuthorizedPasteInstance(const int & NbSelectedInstances, ObjID * iSelectedInstances)
{
  Error ier;
  ier.Ier=0;
  ier.ShortMsg=NULL;
  ier.LongMsg=NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("AuthorizedPasteInstance\n");
      printf ("**** Nb selected objects: %i ****\n", NbSelectedInstances);
      for (int i=0;i<NbSelectedInstances;i++)
	{
	  printf("      **** Obj = %i ****\n",i);
	  printf("           Coid   = [%16.16s]\n",iSelectedInstances[i].Xcoid);
	  printf("           Compid = [%16.16s]\n",iSelectedInstances[i].Xcompid);
	  printf("           Caenv  = [%8.8s]\n",iSelectedInstances[i].caenv);
	  printf("           Catab  = [%18.18s]\n",iSelectedInstances[i].catab);
	}
    }
  //--------------------------
  return ier;
}

Error CATDMUsrMethod::AuthorizedMove(const ObjID& ilink)
{
  Error ier;
  ier.Ier=0;
  ier.ShortMsg=NULL;
  ier.LongMsg=NULL;
  //  ier.Ier=16308;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("AuthorizedMove [%s], [%s], [%s], [%s]\n",
	      ilink.caenv, ilink.catab, ilink.Xcoid, ilink.Xcompid);
    }
  //--------------------------
  return ier;
}

Error CATDMUsrMethod::AuthorizedCheckOut(const ObjID& iObj)
{
  Error ier;
  ier.Ier=0;
  ier.ShortMsg=NULL;
  ier.LongMsg=NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("AuthorizedCheckOut [%s], [%s], [%s], [%s]\n",
	      iObj.caenv, iObj.catab, iObj.Xcoid, iObj.Xcompid);
    }
  //--------------------------
  return ier;
}

Error CATDMUsrMethod::AuthorizedCheckIn(const ObjID& iObj)
{
  Error ier;
  ier.Ier=0;
  ier.ShortMsg=NULL;
  ier.LongMsg=NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("AuthorizedCheckIn [%s], [%s], [%s], [%s]\n",
	      iObj.caenv, iObj.catab, iObj.Xcoid, iObj.Xcompid);
    }
  //--------------------------
  return ier;
}

Error CATDMUsrMethod::AuthorizedDelete(const ObjID& iObj)
{
  Error ier;
  ier.Ier=0;
  ier.ShortMsg=NULL;
  ier.LongMsg=NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("AuthorizedDelete [%s], [%s], [%s], [%s]\n",
	      iObj.caenv, iObj.catab, iObj.Xcoid, iObj.Xcompid);
    }
  //--------------------------
  return ier;
}

Error CATDMUsrMethod::AuthorizedCreate(const ObjID& iObj)
{
  Error ier;
  ier.Ier=0;
  ier.ShortMsg=NULL;
  ier.LongMsg=NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("AuthorizedCreate [%s], [%s], [%s], [%s]\n",
	      iObj.caenv, iObj.catab, iObj.Xcoid, iObj.Xcompid);
    }
  //--------------------------
  return ier;
}

Error CATDMUsrMethod::AuthorizedDefineInstance(const int & NbSelectedInstances, ObjID * iObj)
{
  Error ier;
  ier.Ier=0;
  ier.ShortMsg=NULL;
  ier.LongMsg=NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("AuthorizedDefineInstance\n");
      printf ("**** Nb selected objects: %i ****\n", NbSelectedInstances);
      for (int i=0;i<NbSelectedInstances;i++)
	{
	  printf("      **** Obj = %i ****\n",i);
	  printf("           Coid   = [%16.16s]\n",iObj[i].Xcoid);
	  printf("           Compid = [%16.16s]\n",iObj[i].Xcompid);
	  printf("           Caenv  = [%8.8s]\n",iObj[i].caenv);
	  printf("           Catab  = [%18.18s]\n",iObj[i].catab);
	}
    }
  //--------------------------
  return ier;
}

Error CATDMUsrMethod::AuthorizedDefineInstance(const ObjID &iRootObj, const int &iLevel, double *iLinksId)
{
  Error ier;
  ier.Ier=0;
  ier.ShortMsg=NULL;
  ier.LongMsg=NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("AuthorizedDefineInstance\n");
      printf("      **** RootObj = ****\n");
      printf("           Coid   = [%16.16s]\n",iRootObj.Xcoid);
      printf("           Compid = [%16.16s]\n",iRootObj.Xcompid);
      printf("           Caenv  = [%8.8s]\n",iRootObj.caenv);
      printf("           Catab  = [%18.18s]\n",iRootObj.catab);
      for (int i=0;i<iLevel;i++)
	{
	  printf("      **** Level = %d ****\n",i);
	  double clink = iLinksId[i];
	  unsigned long *p = (unsigned long *)&clink;
	  printf("           Link Compid = [%.8X%.8X]\n", *p, *(p+1));
	}
    }
  //--------------------------
  return ier;
}


Error CATDMUsrMethod::AuthorizedEditInstance(const int & NbSelectedInstances, ObjID * iObj)
{
  Error ier;
  ier.Ier=0;
  ier.ShortMsg=NULL;
  ier.LongMsg=NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("AuthorizedEditInstance\n");
      printf ("**** Nb selected objects: %i ****\n", NbSelectedInstances);
      for (int i=0;i<NbSelectedInstances;i++)
	{
	  printf("      **** Obj = %i ****\n",i);
	  printf("           Coid   = [%16.16s]\n",iObj[i].Xcoid);
	  printf("           Compid = [%16.16s]\n",iObj[i].Xcompid);
	  printf("           Caenv  = [%8.8s]\n",iObj[i].caenv);
	  printf("           Catab  = [%18.18s]\n",iObj[i].catab);
	}
    }
  //--------------------------
  return ier;
}

Error CATDMUsrMethod::AuthorizedUpdate(const ObjID& iObj)
{
  Error ier;
  ier.Ier=0;
  ier.ShortMsg=NULL;
  ier.LongMsg=NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("AuthorizedUpdate [%s], [%s], [%s], [%s]\n",
	      iObj.caenv, iObj.catab, iObj.Xcoid, iObj.Xcompid);
    }
  //--------------------------
  return ier;
}

Error CATDMUsrMethod::AuthorizedReplace(const ObjID& iFather,const ObjID& iLink,
					const ObjID& iChild,const ObjID& iNewChild)
{
  Error ier;
  ier.Ier=0;
  ier.ShortMsg=NULL;
  ier.LongMsg=NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("AuthorizedReplace :\n");
      printf ("\tFather :[%s], [%s], [%s], [%s]\n",
	      iFather.caenv, iFather.catab, iFather.Xcoid, iFather.Xcompid);
      printf ("\tLink :[%s], [%s]\n",
	      iLink.Xcoid, iLink.Xcompid);
      printf ("\tChild :[%s], [%s], [%s], [%s]\n",
	      iChild.caenv, iChild.catab, iChild.Xcoid, iChild.Xcompid);
      printf ("\tNew child : :[%s], [%s], [%s], [%s]\n",
	      iNewChild.caenv, iNewChild.catab, iNewChild.Xcoid, iNewChild.Xcompid);
    }
  //--------------------------
  return ier;
}

Error CATDMUsrMethod::AuthorizedQueryOnInstance(const ObjID& iInstance)
{
  Error ier;
  ier.Ier=0;
  ier.ShortMsg=NULL;
  ier.LongMsg=NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("AuthorizedQueryOnInstance [%s], [%s], [%s], [%s]\n",
	      iInstance.caenv, iInstance.catab, iInstance.Xcoid, iInstance.Xcompid);
    }
  //--------------------------
  return ier;
}

int CATDMUsrMethod::AuthorizedCATIASave(const ObjID& iObj)
{
  int ier=0;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("AuthorizedCATIASave [%s], [%s], [%s], [%s]\n",
	      iObj.caenv, iObj.catab, iObj.Xcoid, iObj.Xcompid);
    }
  //--------------------------
  return ier;
}

Error CATDMUsrMethod::AuthorizedCATIAV5Save(const ObjID& iObj)
{
  Error ier;
  ier.Ier=0;
  ier.ShortMsg=NULL;
  ier.LongMsg=NULL;

  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("AuthorizedCATIAV5Save [%s], [%s], [%s], [%s]\n",
	      iObj.caenv, iObj.catab, iObj.Xcoid, iObj.Xcompid);
    }
  //--------------------------

  return ier;
}

Error CATDMUsrMethod::AuthorizedPromotion(const ObjID& iObj,
					  const char* iOldStatuss,
					  const char* iNewStatus)
{
  Error ier;
  ier.Ier=0;
  ier.ShortMsg=NULL;
  ier.LongMsg=NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("AuthorizedPromotion(%s,%s) [%s], [%s], [%s], [%s]\n",iOldStatuss,iNewStatus,
	      iObj.caenv, iObj.catab, iObj.Xcoid, iObj.Xcompid);
    }
  //--------------------------
  return ier;
}

Error CATDMUsrMethod::AuthorizedReassign(const ObjID& iObj,
					 const char iUser[9], 
					 const char iOrg[11],
					 const char iSite[11])
{
  Error ier;    
  ier.Ier=0;
  ier.ShortMsg = NULL;
  ier.LongMsg = NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("AuthorizedReassign\n");
    }
  
  return ier;
					
}

void CATDMUsrMethod::GetInputListOfValues(const char* iCatab,
					  const char* iAttribute,
					  int* oAuthorized, 
					  int* oNbValues, char*** oValues)
// *oAuthorized=-1; //Help values without default value
// *oAuthorized=1;  //Authorized values without default value
// *oAuthorized=-2; //Help values with first value as default value
// *oAuthorized=2;  //Authorized values with first value as default value
{
  //--------------------------
  //Your code !!

  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("GetInputListOfValues\n");
    }
}

Error CATDMUsrMethod::ComputeUserValuesBeforeWrite(const int iUpdate,
						   const char* const iCaenv, 
						   const char* const iCatab,
						   int *iNbAttPart, int *iNbAttObj,
						   char ***ioAttNames, char ***ioAttVals)
{    
  Error ier;
  ier.Ier=0;
  ier.ShortMsg=NULL;
  ier.LongMsg=NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("ComputeUserValuesBeforeWrite  environmment = [%s]    Update =[%i]\n", iCaenv,iUpdate);
      for(int i=0; i<*iNbAttPart; i++)
	  printf ("    PART_LIST.%s = [%s]\n",(*ioAttNames)[i],(*ioAttVals)[i]);
      for(i=*iNbAttPart; i<*iNbAttPart+*iNbAttObj; i++)
	  printf ("    %s.%s = [%s]\n",iCatab,(*ioAttNames)[i],(*ioAttVals)[i]);
    }
  //--------------------------
  return ier;
}

Error CATDMUsrMethod::ComputeValuesBeforeWrite(const int iUpdate,
					       const char* const iCaenv, 
					       const char* const iCatab,
					       int *iNbAttPart, int *iNbAttObj,
					       char ***ioAttNames, char ***ioAttVals)
{    
  Error ier;
  ier.Ier=0;
  ier.ShortMsg=NULL;
  ier.LongMsg=NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("ComputeValuesBeforeWrite  environmment = [%s]    Update =[%i]\n", iCaenv,iUpdate);
      for(int i=0; i<*iNbAttPart; i++)
	  printf ("    PART_LIST.%s = [%s]\n",(*ioAttNames)[i],(*ioAttVals)[i]);
      for(i=*iNbAttPart; i<*iNbAttPart+*iNbAttObj; i++)
	  printf ("    %s.%s = [%s]\n",iCatab,(*ioAttNames)[i],(*ioAttVals)[i]);
    }
  //--------------------------
  return ier;
}

Error CATDMUsrMethod::CheckLFBeforeWrite(const char* const iCaenv, 
					 const char* const iCatab,
					 const int iSource,
					 char **ioPath,
					 ObjID *ioObj,
					 char** ioRepository)
{
  Error ier;
  ier.Ier=0;
  ier.ShortMsg=NULL;
  ier.LongMsg=NULL;

  //--------------------------
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("CheckLFBeforeWrite  environmment : [%s]    Table :[%s]\n", iCaenv,iCatab);
      printf("\tiSource = %d\n",iSource);
      switch (iSource)
	{
	case 1 :
	  printf ("\t File : [%s]\n", *ioPath);
	  break;
	  
	case 2 :
	  printf ("\t CATIA\n");
	  break;
	  
	case 3:
	  printf ("\t New one\n");
	  break;
	  
	case 4:
	   printf ("\t Object :  [%s], [%s], [%s], [%s]\n",
		   ioObj->caenv, ioObj->catab, ioObj->Xcoid, ioObj->Xcompid);
	  break;

	case 5 :
	  printf ("\t Document\n");
	  break;

	case 7 :
	  printf ("\t Deneb product\n");
	  break;
	}
       printf ("\n\t Repository : [%s]\n", *ioRepository);
    }

  //--------------------------
  return ier;
}

void CATDMUsrMethod::GetNewVersionListOfValues(const ObjID& iObj         ,
                                               const char*  iMaturity    ,
                                               const char*  iLastVer     ,
                                               const char*  iCurVer      ,
                                               int*         oNbVerValues ,
                                               char***      oVerValues   ,
                                               int*         oIsAuthorized)
{
  *oNbVerValues=0;
  *oVerValues=NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
  {
      printf("GetNewVersionListOfValues(%s, %s) [%s], [%s], [%s], [%s]\n", iLastVer, iCurVer,
             iObj.caenv, iObj.catab, iObj.Xcoid, iObj.Xcompid);
      for(int i=0; i<*oNbVerValues; i++)
	  printf ("    [%d] = [%s]\n", i, (*oVerValues)[i]);
  }
  //--------------------------
}

void CATDMUsrMethod::GetNewRevisionListOfValues(const ObjID& iObj        ,
                                                const char*  iMaturity   ,
                                                const char*  iLastRev    ,
                                                const char*  iCurRev     ,
                                                int*         oNbRevValues,
                                                char***      oRevValues  ,
                                                int*         oAuthorized )
{
  *oNbRevValues=0;
  *oRevValues=NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
  {
      printf("GetNewRevisionListOfValues(%s, %s) [%s], [%s], [%s], [%s]\n", iLastRev, iCurRev,
             iObj.caenv, iObj.catab, iObj.Xcoid, iObj.Xcompid);
      for(int i=0; i<*oNbRevValues; i++)
	  printf ("    [%d] = [%s]\n", i, (*oRevValues)[i]);
  }
  //--------------------------
}

Error CATDMUsrMethod::IsAValidNewVersion(const ObjID&       iObj     ,
                                         const char*        iMaturity,
                                         const char*        iPrevVer ,
                                         const char*        iNewVer  ,
                                         const RevVerAction iAction  )
{
  Error ier;
  ier.Ier=0;
  ier.ShortMsg=NULL;
  ier.LongMsg=NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
  {
      printf("IsAValidNewVersion(%s, %s) [%s], [%s], [%s], [%s]\n", iPrevVer, iNewVer,
             iObj.caenv, iObj.catab, iObj.Xcoid, iObj.Xcompid);
  }
  //--------------------------
  return ier;
}

Error CATDMUsrMethod::IsAValidNewRevision(const ObjID&       iObj     ,
                                          const char*        iMaturity,
                                          const char*        iPrevRev ,
                                          const char*        iNewRev  ,
                                          const RevVerAction iAction  )
{
  Error ier;
  ier.Ier=0;
  ier.ShortMsg=NULL;
  ier.LongMsg=NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
  {
      printf("IsAValidNewRevision(%s, %s) [%s], [%s], [%s], [%s]\n", iPrevRev, iNewRev,
             iObj.caenv, iObj.catab, iObj.Xcoid, iObj.Xcompid);
  }
  //--------------------------
  return ier;
}

Error CATDMUsrMethod::IsAValidDuplicatedRevision(const ObjID& iObj    ,
                                                 const char*  iPrevRev,
                                                 const char*  iNewRev )
{
  Error ier;
  ier.Ier=0;
  ier.ShortMsg=NULL;
  ier.LongMsg=NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
  {
      printf("IsAValidDuplicatedRevision(%s, %s) [%s], [%s], [%s], [%s]\n", iPrevRev, iNewRev,
             iObj.caenv, iObj.catab, iObj.Xcoid, iObj.Xcompid);
  }
  //--------------------------
  return ier;
}

Error CATDMUsrMethod::IsAValidRevisionVersion(const ObjID& iObj         ,
                                              const char   iVersion[4]  ,
                                              const char   iOldVerRev[4], 
                                              const char   iNewVerRev[4])
{
  Error ier;    
  ier.Ier=0;
  ier.ShortMsg = NULL;
  ier.LongMsg = NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("IsAValidRevisionVersion\n");
    }
  //--------------------------
  return ier;
}

Error CATDMUsrMethod::AuthorizedNewVersion(const ObjID& iObj       ,
                                           const int iNewPartNumber)
{
  Error ier;
  ier.Ier=0;
  ier.ShortMsg=NULL;
  ier.LongMsg=NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("AuthorizedNewVersion(%i) [%s], [%s], [%s], [%s]\n",iNewPartNumber,
	      iObj.caenv, iObj.catab, iObj.Xcoid, iObj.Xcompid);
    }
  //--------------------------
  return ier;
}

Error CATDMUsrMethod::AuthorizedNewRevision(const ObjID& iObj)
{
  Error ier;
  ier.Ier=0;
  ier.ShortMsg=NULL;
  ier.LongMsg=NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("AuthorizedNewRevision [%s], [%s], [%s], [%s]\n",
	      iObj.caenv, iObj.catab, iObj.Xcoid, iObj.Xcompid);
    }
  //--------------------------
  return ier;
}

int CATDMUsrMethod::IsMassiveDuplicateAllowed(const ObjID& iObj, int iTypeOfCheck)
{
  int isAllowed = 1;

  Error ier;
  ier.Ier=0;
  ier.ShortMsg=NULL;
  ier.LongMsg=NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("IsMassiveDuplicateAllowed [%s], [%s], [%s], [%s], [%u]\n",
	      iObj.caenv, iObj.catab, iObj.Xcoid, iObj.Xcompid, iTypeOfCheck);
    }
  //--------------------------
  return isAllowed;
}

Error CATDMUsrMethod::DuplicateLink(const ObjID& iOld, const ObjID& iNew)
{
  Error ier;
  ier.Ier=0;
  ier.ShortMsg=NULL;
  ier.LongMsg=NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("DuplicateLink\n");
    }
  //--------------------------
  return ier;
}

void CATDMUsrMethod::Replace(const ObjID& iOldLink, const ObjID& iNewLink)
{
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("Replace\n");
    }
  //--------------------------
}

Error CATDMUsrMethod::GetReplaceCustomParts(const ObjID& iFather           ,
                                            const ObjID& iLink             ,
                                            const ObjID& iChild            ,
                                            int&         oNbCandidateChild ,
                                            ObjID*&      oCandidateChildren)
{
  Error ier;
  ier.Ier=0;
  ier.ShortMsg=NULL;
  ier.LongMsg=NULL;

  oNbCandidateChild=0;
  oCandidateChildren=NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("GetReplaceCustomParts\n");
    }

  return(ier);
}

Error CATDMUsrMethod::GetRevisions(const char  *   iType,
                                   const ObjID  &  iInitialObject,
                                         int    &  oSearchedPerformed,
                                         int    &  oNbRevisions,
                                         ObjID *&  oRevisions)
{
  Error ier;
  ier.Ier=0;
  ier.ShortMsg=NULL;
  ier.LongMsg=NULL;

  oSearchedPerformed = 0;
  oNbRevisions       = 0;
  oRevisions         = NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("GetRevisions\n");
    }

  return(ier);
}

Error CATDMUsrMethod::BeforeCreateUserAction(const char* const iCaenv, const char* const iObject)
{
  Error ier;    
  ier.Ier=0;
  ier.ShortMsg = NULL;
  ier.LongMsg = NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("BeforeCreateUserAction : [%s], [%s]\n",iCaenv,iObject);
    }
  
  return ier;
}


Error CATDMUsrMethod::AfterWriteUserAction(const ObjID& iObj)
{
  Error ier;    
  ier.Ier=0;
  ier.ShortMsg = NULL;
  ier.LongMsg = NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("AfterWriteUserAction\n");
    }
  
  return ier;
}

Error CATDMUsrMethod::AfterCommitUserAction()
{
  Error ier;    
  ier.Ier=0;
  ier.ShortMsg = NULL;
  ier.LongMsg = NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("AfterCommitUserAction\n");
    }
  
  return ier;
}

void CATDMUsrMethod::AfterRollbackUserAction()
{
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("AfterRollbackUserAction\n");
    }
}

char* CATDMUsrMethod::GetBGColorForPrint(const char* iCatab,
					 const char* iAttribute,
					 const char* iValue)
{
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("GetBGColorForPrint\n");
    }
  char *sortie=NULL;
  if(strcmp(iValue,"10%")==0) sortie=strdup("#00FF00");
  else if(strcmp(iValue,"100%")==0 || strcmp(iValue,"RELEASED")==0) sortie=strdup("#FF0000");
  
  return(sortie);
}

Error CATDMUsrMethod::FormatConfigHandlerId(const ObjID& iObj, 
					    const char* iCHUserId, 
					    char **oCHNewId)
{
  Error ier;    
  ier.Ier=0;
  ier.ShortMsg = NULL;
  ier.LongMsg = NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("FormatConfigHandlerId\n");
    }
  
  return ier;
}

Error CATDMUsrMethod::OptionListToDisplay(const char *iUser, 
					  const char *iRole, 
					  const char *iOrg, 
					  const ObjID& iObj, 
					  const int iNbOpt, 
					  int &oNbOpt, 
					  char ***ioOptNameList, 
					  char ***ioOptTypeList, 
					  char ***ioOptDescList)
{
  Error ier;    
  ier.Ier=0;
  ier.ShortMsg = NULL;
  ier.LongMsg = NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("OptionListToDisplay\n");
    }

  return ier;
}

Error CATDMUsrMethod::AuthorizedEffModification(const char *iUser,
						const char *iRole,
						const char *iOrg,
						const ObjID& iLink,                            
						const EffParam& iOldEff,                            
						const EffParam& iNewEff)                          
{
  Error ier;    
  ier.Ier=0;
  ier.ShortMsg = NULL;
  ier.LongMsg = NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("AuthorizedEffModification\n");
    }
  return ier;
}

Error CATDMUsrMethod::AuthorizedEffDefinition(const char *iUser,
					      const char *iRole,
					      const char *iOrg,
					      const ObjID& iLink,                            
					      const int iNbEffParam, 
					      EffParam *iEffParamPtr)                          
{
  Error ier;    
  ier.Ier=0;
  ier.ShortMsg = NULL;
  ier.LongMsg = NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("AuthorizedEffDefinition\n");
    }
  return ier;
}



Error CATDMUsrMethod::AuthorizedProposedEffModification(const char *iUser, const char *iRole, const char *iOrg, 
							const char *iActionName,                            
							const int iNbEffParam, EffParam *iEffParamPtr)                          
{
  Error ier;    
  ier.Ier=0;
  ier.ShortMsg = NULL;
  ier.LongMsg = NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("AuthorizedProposedEffModification\n");
    }
  return ier;
}

Error CATDMUsrMethod::AuthorizedDuplicatedRepresentations(int*    ioNbObj,
                                                          ObjID** ioObjs )
{
  Error ier;    
  ier.Ier=0;
  ier.ShortMsg = NULL;
  ier.LongMsg = NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("AuthorizedDuplicatedRepresentations\n");
    }
  return ier;
}

Error CATDMUsrMethod::AuthorizedDuplicatedAssemblyLinks(int*    ioNbObj,
							ObjID** ioObjs )
{
  Error ier;    
  ier.Ier=0;
  ier.ShortMsg = NULL;
  ier.LongMsg = NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("AuthorizedDuplicatedAssemblyLinks\n");
      printf ("\tNb links: %d \n", *ioNbObj);
      for (int i=0;i<*ioNbObj;i++)
	{
	  printf("\tObj = %d\n",i);
	  printf("\t\tCoid   = [%16.16s]\n",(*ioObjs)[i].Xcoid);
	  printf("\t\tCompid = [%16.16s]\n",(*ioObjs)[i].Xcompid);
	  printf("\t\tCaenv  = [%.8s]\n",(*ioObjs)[i].caenv);
	  printf("\t\tCatab  = [%.18s]\n",(*ioObjs)[i].catab);
	}
    }
  
  return ier;
}


Error CATDMUsrMethod::ConfigHandlerControl(const char *iUser, 
                                           const char *iRole, 
                                           const char *iOrg, 
                                           const ObjID&   iProduct,    
                                           const char*    iCHName, 
                                           const CHAction iAction) 
{
  Error ier;    
  ier.Ier=0;
  ier.ShortMsg = NULL;
  ier.LongMsg = NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("ConfigHandlerControl\n");
    }

  return ier;
}

void CATDMUsrMethod::ConfigHandlerVisibilityControl(const char*  iUser, 
						    const char*  iRole, 
						    const char*  iOrg, 
						    const ObjID& iProduct,    
						    const char*  iCHName, 
						    int&         oAuthorizedVisibility, 
						    int&         oDefaultVisibility)
{
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("ConfigHandlerVisibilityControl\n");
    }
}

Error CATDMUsrMethod::DictionaryControl(const char *iUser, const char *iRole, const char *iOrg, 
					const ObjID&     iProduct,    
					const char*      iOptionName, 
					const char*      iCategoryName, 
					const char*      iCategoryOwner, 
					const char*      iCategoryOrg, 
					const DicoAction iAction)                      
{
  Error ier;    
  ier.Ier=0;
  ier.ShortMsg = NULL;
  ier.LongMsg = NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("DictionaryControl\n");
    }

  return ier;
}

Error CATDMUsrMethod::DictionaryRulesControl(const char *iUser, const char *iRole, const char *iOrg, 
					     const ObjID&          iProduct,    
					     const char*           iRuleId, 
					     const char*           iRuleOwner, 
					     const char*           iRuleOrg, 
					     const DicoRulesAction iAction)                      
{
  Error ier;    
  ier.Ier=0;
  ier.ShortMsg = NULL;
  ier.LongMsg = NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("DictionaryRulesControl\n");
    }

  return ier;
}

Error CATDMUsrMethod::MilestonesControl(const char *iUser, const char *iRole, const char *iOrg, 
					const ObjID&           iProduct,    
					const char*            iProgramName, 
					const char*            iMilestoneName, 
					const int              iMilestoneType, 
					const MilestonesAction iAction)                      
{
  Error ier;    
  ier.Ier=0;
  ier.ShortMsg = NULL;
  ier.LongMsg = NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("MilestonesControl\n");
    }

  return ier;
}

Error CATDMUsrMethod::CommitModeControl(const char *iUser, const char *iRole, const char *iOrg, 
					int& oCommitModeToDisable)
{
  Error ier;    
  ier.Ier=0;
  ier.ShortMsg = NULL;
  ier.LongMsg = NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("CommitModeControl\n");
    }

  return ier;
}


Error CATDMUsrMethod::OpenGraph(const int iGraphNum, const ObjID&   iPart)
{
  Error ier;    
  ier.Ier=0;
  ier.ShortMsg = NULL;
  ier.LongMsg = NULL;

  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("OpenGraph [%s], [%s], [%s], [%s]\n",
	      iPart.caenv, iPart.catab, iPart.Xcoid, iPart.Xcompid);
    }

  return ier;
}
void CATDMUsrMethod::CloseGraph(const int iGraphNum, const ObjID& iPart)
{
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("CloseGraph [%s], [%s], [%s], [%s]\n",
	      iPart.caenv, iPart.catab, iPart.Xcoid, iPart.Xcompid);
    }
}

void CATDMUsrMethod::ExitVPMConnectedSession()
{
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("ExitVPMConnectedSession\n");
    }
}

Error CATDMUsrMethod::OpenObject(const char *iViewer,const int iNbObj,ObjID *iObj)
{
   Error ier;    
   ier.Ier=0;
   ier.ShortMsg = NULL;
   ier.LongMsg = NULL;

  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("OpenObject : Viewer = %s\n",iViewer);
      for(int i=0;i<iNbObj;i++)
	printf("\tObject : [%s], [%s], [%s], [%s]\n",
	       iObj[i].caenv, iObj[i].catab, iObj[i].Xcoid, iObj[i].Xcompid);
    }
   
   return ier;
}


void CATDMUsrMethod::PSNCutCB(const CATLISTP(CATVPMNode) &iObjs)
{
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    printf ("PSNCutCB\n");
}

void CATDMUsrMethod::PSNCopyCB(const CATLISTP(CATVPMNode) &iObjs)
{
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    printf ("PSNCopyCB\n");
}


void CATDMUsrMethod::PSNPasteCB(CATVPMNode *iObj, SdaiAppInstance *iLink, SdaiAppInstance *iPaste)
{
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    printf ("PSNPasteCB\n");
}
//
//  Backup management
//
int CATDMUsrMethod::BckModR(const ObjID& iObj, const int iNb, char **iRType
                           ,int *oNb, int **oLnkOI, char ***oRType)
{
  //--------------------------
  //Your code !!
  *oNb      = iNb;
  (*oRType) = new char* [*oNb];
  (*oLnkOI) = new int [*oNb];

  for (int i = 0; i < *oNb; i ++)
  {
    (*oRType)[i] = new char [strlen(iRType[i])+1];
    strcpy((*oRType)[i], iRType[i]);           

//    (*oRType)[i] = new char [strlen(iRType[i]+11)+1]; //without 'old.backup.'
//    strcpy((*oRType)[i], iRType[i]+11);               //without 'old.backup.'

    (*oLnkOI)[i] = i;                             
  }

  if(getenv("TRACEUSRMETHOD"))
  {
    printf("BckModR [%s], [%s], [%s], [%s]\n"
          ,iObj.caenv, iObj.catab, iObj.Xcoid, iObj.Xcompid);

    for(i=0; i<iNb; i++)
      printf("in  REPRES_TYPE[%d] = [%s]\n",i,iRType[i]);

    for(i=0; i<*oNb; i++)
      printf("out  REPRES_TYPE[%d] = [%s] -> [%d][%s]\n"
            ,i,(*oRType)[i],(*oLnkOI)[i],iRType[(*oLnkOI)[i]]);
  }

  return 0;
}


Error CATDMUsrMethod::UpdateObject(const ObjID& iObj,
				   int *ioNbAtt,
				   char ***ioAttNames, char ***ioAttVals)
{
  Error ier;    
  ier.Ier=0;
  ier.ShortMsg = NULL;
  ier.LongMsg = NULL;
  
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("UpdateObject : \n");
      printf("\tObject : [%s], [%s], [%s], [%s]\n",
	     iObj.caenv, iObj.catab, iObj.Xcoid, iObj.Xcompid);
      for(int i=0; i<*ioNbAtt; i++)
	  printf ("\t%s = [%s]\n",(*ioAttNames)[i],(*ioAttVals)[i]);
    }

   return ier;
}

void CATDMUsrMethod::ExecuteCommand(int &ioLng, char *& ioMsg)
{
  Error ier;    
  ier.Ier=0;
  ier.ShortMsg = NULL;
  ier.LongMsg = NULL;

  if(getenv("TRACEUSRMETHOD"))
  {
    printf ("ExecuteCommand\n");
  }
  ISPPS9DMUsrMethod::ExecuteCommand (ioLng, ioMsg);
}

Error CATDMUsrMethod::AuthorizedMMLAccept(const ObjID& iFrom, const ObjID& iTo)
{
  Error ier;
  ier.Ier=0;
  ier.ShortMsg=NULL;
  ier.LongMsg=NULL;
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("AuthorizedMMLAccept:\n");fflush(stdout);
      printf ("   iFrom: [%s]-[%s]-[%s]-[%s]\n",
	      iFrom.caenv, iFrom.catab, iFrom.Xcoid, iFrom.Xcompid);
      printf ("   iTo: [%s]-[%s]-[%s]-[%s]\n",
	      iTo.caenv, iTo.catab, iTo.Xcoid, iTo.Xcompid);
      fflush(stdout);
    }
  
  //  ier.Ier=1111;
  //  ier.ShortMsg=strdup("Short message!");
  //  ier.LongMsg=strdup("Long message!");
  //--------------------------
  return ier;
}


Error CATDMUsrMethod::AuthorizedDuplicate(const ObjID& iObj)
{
  Error ier;    
  ier.Ier=0;
  ier.ShortMsg = NULL;
  ier.LongMsg = NULL;
  
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("AuthorizedDuplicate : \n");
      printf("\tObject : [%s], [%s], [%s], [%s]\n",
	     iObj.caenv, iObj.catab, iObj.Xcoid, iObj.Xcompid);
    }

   return ier;
}


Error CATDMUsrMethod::UpdateConfigViewValuesForSecurity(const char* const iCaenv, 
							const char* const iCatab,
							int *ioNbAtt,
							char ***ioAttNames, 
							char ***ioAttVals,
							int *ioModified)
{
  Error ier;    
  ier.Ier=0;
  ier.ShortMsg = NULL;
  ier.LongMsg = NULL;
  
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("UpdateConfigViewValuesForSecurity : \n");

      printf ("environmment = [%s]    table =[%s]\n", iCaenv,iCatab);
      for(int i=0; i<*ioNbAtt; i++)
	  printf ("    %s = [%s]\n",(*ioAttNames)[i],(*ioAttVals)[i]);
    }

  return ier;
}

Error CATDMUsrMethod::GetLastSuccessfullExportTimeStamp(const ObjID& iObj,
						   const char *iTargetSite,
						   char *oLastTS)
{
  Error ier;    
  ier.Ier=0;
  ier.ShortMsg = NULL;
  ier.LongMsg = NULL;
  
  //--------------------------
  //Your code !!
  strcpy(oLastTS,"1970-01-01-01.00.00.000000");
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("GetLastSuccessfullExportTimeStamp : \n");

      printf("\tObject : [%s], [%s], [%s], [%s]\n",
	     iObj.caenv, iObj.catab, iObj.Xcoid, iObj.Xcompid);
      printf("\tTarget Site : [%s]\n",iTargetSite);
      printf("\tLast Successfull Export TS : [%s]\n",oLastTS);
    }
 
  return ier;
}

Error CATDMUsrMethod::AuthorizedCreateZone(const ObjID& iObj)
{
  Error ier;    
  ier.Ier=0;
  ier.ShortMsg = NULL;
  ier.LongMsg = NULL;
  
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("AuthorizedCreateZone : \n");

      printf("\tObject : [%s], [%s], [%s], [%s]\n",
	     iObj.caenv, iObj.catab, iObj.Xcoid, iObj.Xcompid);
    }
 
  return ier;
}


Error CATDMUsrMethod::AuthorizedDeleteZone(const ObjID& iObj)
{
  Error ier;    
  ier.Ier=0;
  ier.ShortMsg = NULL;
  ier.LongMsg = NULL;
  
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("AuthorizedDeleteZone : \n");

      printf("\tObject : [%s], [%s], [%s], [%s]\n",
	     iObj.caenv, iObj.catab, iObj.Xcoid, iObj.Xcompid);
    }
 
  return ier;
}

Error CATDMUsrMethod::BeforePartIsImported(const ObjID& iObj, int& abortImport)
{
  Error ier;    
  ier.Ier=0;
  ier.ShortMsg = NULL;
  ier.LongMsg = NULL;
  
  //--------------------------
  //Your code !!
  abortImport=0;
  
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("BeforePartIsImported : \n");

      printf("\tObject : [%s], [%s], [%s], [%s]\n",
	     iObj.caenv, iObj.catab, iObj.Xcoid, iObj.Xcompid);
    }
 
  return ier;
}

    
Error CATDMUsrMethod::AfterPartIsImported(const ObjID& iObj)
{
  Error ier;    
  ier.Ier=0;
  ier.ShortMsg = NULL;
  ier.LongMsg = NULL;
  
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("AfterPartIsImported : \n");

      printf("\tObject : [%s], [%s], [%s], [%s]\n",
	     iObj.caenv, iObj.catab, iObj.Xcoid, iObj.Xcompid);
    }
 
  return ier;
}


Error CATDMUsrMethod::BeforeAddPartToQueue(const ObjID& iObj, char *ioTargetSite, char *ioUser, char *ioOrg, int &ioDoTransfertToSite)
{
  Error ier;    
  ier.Ier=0;
  ier.ShortMsg = NULL;
  ier.LongMsg = NULL;
  
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("BeforePartIsExported : \n");

      printf("\tObject : [%s], [%s], [%s], [%s]\n",
	     iObj.caenv, iObj.catab, iObj.Xcoid, iObj.Xcompid);
      printf("\tTarget Site: [%s]\tUser: [%s]\tOrg: [%s]\n", 
             ioTargetSite, ioUser, ioOrg);

      printf("\tTransfert %s be done\n", ioDoTransfertToSite!=0?"will":"will NOT");
      
    }
    
  return ier;
} 

Error CATDMUsrMethod::AuthorizedNewDocumentV5(const ObjID *iObj, int &oDefaultValue, int &oEditable)
{

  Error ier;    
  ier.Ier=0;
  ier.ShortMsg = NULL;
  ier.LongMsg = NULL;
  
  //--------------------------
  //Your code !!
  if(getenv("TRACEUSRMETHOD"))
    {
      printf ("AuthorizedNewDocumentV5 : \n");
      if (iObj!=NULL)
	{
	  printf("\tObject : [%s], [%s], [%s], [%s]\n",
		 (*iObj).caenv, (*iObj).catab, (*iObj).Xcoid, (*iObj).Xcompid);
	}
    }
  

  return ier;
}
